@@ -9,6 +9,7 @@
 import os
 import sys
 import doctest
+import resource
 
 flags = doctest.ELLIPSIS
 
@@ -24,6 +25,14 @@ for typ in [AlarmInterrupt, SignalError]:
     typ.__module__ = "__main__"
 
 
+# Limit stack size to avoid errors in stack overflow doctest
+stacksize = 1 << 20
+resource.setrlimit(resource.RLIMIT_STACK, (stacksize, stacksize))
+
+# Disable core dumps
+resource.setrlimit(resource.RLIMIT_CORE, (0, 0))
+
+
 success = True
 for f in filenames:
     print(f)
@@ -39,10 +48,20 @@ for f in filenames:
             if not failures:
                 os._exit(0)
         finally:
-            os._exit(1)
+            os._exit(23)
 
     pid, status = os.waitpid(pid, 0)
     if status != 0:
         success = False
+        if os.WIFEXITED(status):
+            st = os.WEXITSTATUS(status)
+            if st != 23:
+                print("bad exit: {}".format(st))
+        elif os.WIFSIGNALED(status):
+            sig = os.WTERMSIG(status)
+            print("killed by signal: {}".format(sig))
+        else:
+            print("unknown status: {}".format(status))
+
 
 sys.exit(0 if success else 1)
