@@ -228,7 +228,7 @@ static void do_raise_exception(int sig)
         gettimeofday(&raisetime, NULL);
         long delta_ms = (raisetime.tv_sec - sigtime.tv_sec)*1000L + ((long)raisetime.tv_usec - (long)sigtime.tv_usec)/1000;
         PyGILState_STATE gilstate = PyGILState_Ensure();
-        fprintf(stderr, "do_raise_exception(sig=%i)\nPyErr_Occurred() = %p\nRaising Python exception %li ms after signals...\n",
+        fprintf(stderr, "do_raise_exception(sig=%i)\nPyErr_Occurred() = %p\nRaising Python exception %li ms after signal...\n",
             sig, PyErr_Occurred(), delta_ms);
         PyGILState_Release(gilstate);
         fflush(stderr);
@@ -300,6 +300,14 @@ static void setup_cysignals_handlers(void)
     sigaddset(&sigmask_with_sigint, SIGINT);
     sigaddset(&sigmask_with_sigint, SIGALRM);
 
+    /* Allocate alternate signal stack */
+    stack_t ss;
+    ss.ss_size = 1 << 14;
+    ss.ss_sp = malloc(ss.ss_size);
+    if (ss.ss_sp == NULL) {perror("malloc"); exit(1);}
+    ss.ss_flags = 0;
+    if (sigaltstack(&ss, NULL) == -1) {perror("sigaltstack"); exit(1);}
+
     /* Install signal handlers */
     struct sigaction sa;
     memset(&sa, 0, sizeof(sa));
@@ -309,14 +317,17 @@ static void setup_cysignals_handlers(void)
     sigaddset(&sa.sa_mask, SIGINT);
     sigaddset(&sa.sa_mask, SIGALRM);
 
+    /* Handlers for interrupt-like signals */
     sa.sa_handler = cysigs_interrupt_handler;
     if (sigaction(SIGHUP, &sa, NULL)) {perror("sigaction"); exit(1);}
     if (sigaction(SIGINT, &sa, NULL)) {perror("sigaction"); exit(1);}
     if (sigaction(SIGALRM, &sa, NULL)) {perror("sigaction"); exit(1);}
+
+    /* Handlers for critical signals */
     sa.sa_handler = cysigs_signal_handler;
     /* Allow signals during signal handling, we have code to deal with
      * this case. */
-    sa.sa_flags |= SA_NODEFER;
+    sa.sa_flags = SA_NODEFER | SA_ONSTACK;
     if (sigaction(SIGQUIT, &sa, NULL)) {perror("sigaction"); exit(1);}
     if (sigaction(SIGILL, &sa, NULL)) {perror("sigaction"); exit(1);}
     if (sigaction(SIGABRT, &sa, NULL)) {perror("sigaction"); exit(1);}
@@ -400,6 +411,8 @@ static void print_enhanced_backtrace(void)
 /* Print a message s and kill ourselves with signal sig */
 static void sigdie(int sig, const char* s)
 {
+    if (getenv("CYSIGNALS_CRASH_QUIET")) goto dienow;
+
     print_sep();
     print_backtrace();
 
@@ -410,7 +423,8 @@ static void sigdie(int sig, const char* s)
 #else
 #ifndef __APPLE__
     /* See http://trac.sagemath.org/13889 for how Apple screwed this up */
-    print_enhanced_backtrace();
+    if (getenv("CYSIGNALS_CRASH_NDEBUG") == NULL)
+        print_enhanced_backtrace();
 #endif
 #endif
 
@@ -423,6 +437,7 @@ static void sigdie(int sig, const char* s)
         print_sep();
     }
 
+dienow:
     /* Suicide with signal ``sig`` */
     kill(getpid(), sig);
 
